@@ -4,6 +4,7 @@ use warnings;
 use strict;
 
 use POE;
+sub POE::Component::Server::REST::DEBUG() { 1 };
 use POE::Component::Server::REST;
 use HTTP::Status qw(:constants);
 
@@ -14,23 +15,21 @@ use constant {
 
 
 # This is OUR session
-my $name = "THING";
+my $name = "library";
 
 # This is the POE..REST server
 my $service = 'restservice';
 
-my @methods = (
-	'GET/things',
-	'GET/thing',
-	'PUT/thing',
-	'POST/thing',
-	'DELETE/thing',
-);
-
-my @content_types = (
-	"text/xml",
-	"text/yaml",
-);
+my $methods = {
+	'_start'        =>  \&start,
+	'_stop'         =>  \&stop,
+	'GET/author'     =>  \&get_author,
+	'GET/author/douglas/adams' => \&get_special,
+	'POST/author'    =>  \&add_author,
+	'PUT/author'     =>  \&upd_author,
+	'DELETE/author'  =>  \&del_author,
+	'GET/authors'    =>  \&get_authors,
+};
 
 use Sys::Hostname;
 
@@ -41,20 +40,13 @@ my $port = 8081;
 POE::Component::Server::REST->new(
     'ALIAS'         =>      $service,
     'ADDRESS'       =>      $address,
-    'PORT'          =>      8081,
+    'PORT'          =>      $port,
     'HOSTNAME'      =>      $hostname,
+	'CONTENTTYPE'	=>	'text/json',
 );
 
 POE::Session->create(
-    'inline_states' =>      {
-		'_start'        =>	\&start,
-		'_stop'         =>	\&stop,
-		'GET/thing'		=>	\&get_thing,
-		'POST/thing'	=>	\&add_thing,
-		'PUT/thing'		=>	\&upd_thing,
-		'DELETE/thing'	=> 	\&del_thing,
-		'GET/things'	=>	\&get_things,
-    },
+    'inline_states' => $methods    
 );
 
 POE::Kernel->run;
@@ -67,11 +59,11 @@ sub start {
 	my $heap = $_[HEAP];
 
 	# Some preparations
-	$heap->{things} = {
-		1 => 'Foo',
-		2 => 'Bar',
-		3 => 'Slow',
-		4 => 'Joe', 
+	$heap->{authors} = {
+		1 => 'Hemmingway',
+		2 => 'Aldous Huxley',
+		3 => 'George Orwell',
+		4 => 'Humberto Eco', 
 	};
 
 	# Necessary in order to let POE..REST server
@@ -79,7 +71,7 @@ sub start {
 	$kernel->alias_set($name);
 
 	# Register Service Methods
-	foreach my $m (@methods) {
+	foreach my $m (keys %$methods) {
 		$kernel->post( $service, 'ADDMETHOD', $name, $m );
 	}
 
@@ -89,49 +81,62 @@ sub stop {
 	my $kernel = $_[KERNEL];
 
 	# Unregister Service Methods	
-	foreach my $m (@methods) {
+	foreach my $m (keys %$methods) {
 		$kernel->post( $service, 'DELMETHOD', $name, $m );
 	}
 }
 
-# Things
+# Authors
 ####################
 
-sub get_thing {
+sub get_author {
 	my ($kernel, $heap, $response, $key) = @_[KERNEL, HEAP, ARG0, ARG1];
 
 	# Check if exists
-	if( exists($heap->{things}->{$key}) ) {
-		  my $val = $heap->{things}->{$key};
+	if( exists($heap->{authors}->{$key}) ) {
+		  my $val = $heap->{authors}->{$key};
 
 		  # Build YAML response, better if you have objects here which can represent themself in yaml/xml/...
 	      $response->content({ 
-			'thing' => { 
+			'author' => { 
 				'id' => $key, 
 				'name' => $val }, 
 			});
 	      $kernel->post( $service, 'DONE', $response, );
 	      return;		
 	} else {
-		$kernel->post( $service, 'FAULT', $response, HTTP_NOT_FOUND, "NotFound", "Thing $key does not exists."  );
+		$kernel->post( $service, 'FAULT', $response, HTTP_NOT_FOUND, "NotFound", "Author $key does not exists."  );
 		return;	
 	}
 }
 
-sub get_things {
+sub get_authors {
       my ($kernel, $heap, $response, $key) = @_[KERNEL, HEAP, ARG0, ARG1];
 
-	  my $things = [];
-	  while( my ($id, $name) = each %{ $heap->{things} }) {
-			push(@$things, { id => $id, name => $name } );
+	  my $authors = [];
+	  while( my ($id, $name) = each %{ $heap->{authors} }) {
+			push(@$authors, { id => $id, name => $name } );
 	  }
-	  $response->content({ things => $things });
+	  $response->content({ authors => $authors });
       $kernel->post($service, 'DONE', $response);
       return;
 
 }
 
-sub add_thing {
+sub get_special {
+    my ($kernel, $heap, $response, $key) = @_[KERNEL, HEAP, ARG0, ARG1];
+
+	# Build YAML response, better if you have objects here which can represent themself in yaml/xml/...
+	$response->content({
+	'author' => {
+		'id' => 4711,
+		'name' => "Douglas Adams" },
+	});
+	$kernel->post( $service, 'DONE', $response, );
+	return;
+}
+
+sub add_author {
 	my ($kernel, $heap, $response, $key) = @_[KERNEL, HEAP, ARG0, ARG1];
 
 	# POE::Component::Server::REST returns undef if there was an error while parsing the request.
@@ -142,42 +147,42 @@ sub add_thing {
 	}
 
 	# Check structure
-	unless( ref($content) eq 'HASH' and exists($content->{thing}) and exists($content->{thing}->{id}) and exists($content->{thing}->{name}) ) {
+	unless( ref($content) eq 'HASH' and exists($content->{author}) and exists($content->{author}->{id}) and exists($content->{author}->{name}) ) {
         $kernel->post( $service, 'FAULT', $response, HTTP_BAD_REQUEST, "Bad Request", "Unable to validate structure." );
         return;
 	}
 
 	# Check for existence
-	if( exists($heap->{things}->{$key}) ) {
-		$kernel->post( $service, 'FAULT', $response, HTTP_BAD_REQUEST, "Bad Request", "Thing $key exists already." );
+	if( exists($heap->{authors}->{$key}) ) {
+		$kernel->post( $service, 'FAULT', $response, HTTP_BAD_REQUEST, "Bad Request", "Author $key exists already." );
 		return;
 	}
 
-	my $id = $content->{thing}->{id};
-	my $name = $content->{thing}->{name};
+	my $id = $content->{author}->{id};
+	my $name = $content->{author}->{name};
 
 	# Validate extracted field id
 	unless( defined $id ) {
-        $kernel->post( $service, 'FAULT', $response, HTTP_BAD_REQUEST, "Bad Request", "Thing id needs to be set!" );
+        $kernel->post( $service, 'FAULT', $response, HTTP_BAD_REQUEST, "Bad Request", "Author id needs to be set!" );
         return;
 	}
 
 	# Validate extracted field name
     unless( defined $name ) {
-        $kernel->post( $service, 'FAULT', $response, HTTP_BAD_REQUEST, "Bad Request", "Thing name needs to be set!" );
+        $kernel->post( $service, 'FAULT', $response, HTTP_BAD_REQUEST, "Bad Request", "Author name needs to be set!" );
         return;
     }
 
 	# Add it
-	$heap->{things}->{$id} = $name;
+	$heap->{author}->{$id} = $name;
 
 	# Done
-	$kernel->post( $service, 'DONE', $response, "Done", "Added thing $id -> $name"  );
+	$kernel->post( $service, 'DONE', $response, "Done", "Added author $id -> $name"  );
 	return;
 
 }
 
-sub upd_thing {
+sub upd_author {
 	my ($kernel, $heap, $response, $key) = @_[KERNEL, HEAP, ARG0, ARG1];
 
     # Parse xml
@@ -188,63 +193,63 @@ sub upd_thing {
     }
 
 	# Sheck structure
-    unless( exists($content->{thing}) and exists($content->{thing}->{id}) && exists($content->{thing}->{name}) ) {
+    unless( exists($content->{author}) and exists($content->{author}->{id}) && exists($content->{author}->{name}) ) {
         $kernel->post( $service, 'FAULT', $response, HTTP_BAD_REQUEST, "Bad Request", "Unable to validate structure." );
         return;
     }
 
-	# Only proceed if referenced thing does exist
-    unless( exists($heap->{things}->{$key}) ) {
-        $kernel->post( $service, 'FAULT', $response, HTTP_BAD_REQUEST, "Bad Request", "Thing $key does not exist." );
+	# Only proceed if referenced author does exist
+    unless( exists($heap->{authors}->{$key}) ) {
+        $kernel->post( $service, 'FAULT', $response, HTTP_BAD_REQUEST, "Bad Request", "Author $key does not exist." );
         return;
     }
 
-    my $id = $content->{thing}->{id};
-    my $name = $content->{thing}->{name};
+    my $id = $content->{author}->{id};
+    my $name = $content->{author}->{name};
 
     # Validate extracted field id
     unless( defined $id ) {
-        $kernel->post( $service, 'FAULT', $response, HTTP_BAD_REQUEST, "Bad Request", "Thing id needs to be set!" );
+        $kernel->post( $service, 'FAULT', $response, HTTP_BAD_REQUEST, "Bad Request", "Author id needs to be set!" );
         return;
     }
 
     # Validate extracted field name
     unless( defined $name ) {
-        $kernel->post( $service, 'FAULT', $response, HTTP_BAD_REQUEST, "Bad Request", "Thing name needs to be set!" );
+        $kernel->post( $service, 'FAULT', $response, HTTP_BAD_REQUEST, "Bad Request", "Author name needs to be set!" );
         return;
     }
 
-	# Update thing
-    $heap->{things}->{$id} = $name;
+	# Update author
+    $heap->{authors}->{$id} = $name;
 
 	# Done
-	$kernel->post( $service, 'DONE', $response, "Done", "Updated thing $id -> $name"  );
+	$kernel->post( $service, 'DONE', $response, "Done", "Updated author $id -> $name"  );
 	return;
 
 }
 
-sub del_thing {
+sub del_author {
     my ($kernel, $heap, $response, $key) = @_[KERNEL, HEAP, ARG0, ARG1];
 
 	my $params = $response->restbody;
 
 	my $apikey = $response->header('APIKEY');
 	unless( $apikey and $apikey eq "raiNaeR2ohnaefex0AeFaig7gujeiQuai8uezaach4oow5chaize9xuxuleiqu0z" ) {
-		$kernel->post( $service, 'FAULT', $response, HTTP_UNAUTHORIZED, 'Access Denied', "You are not privileged to delete things." );
+		$kernel->post( $service, 'FAULT', $response, HTTP_UNAUTHORIZED, 'Access Denied', "You are not privileged to delete authors." );
 		return;
 	}
 
-    # Check if referenced thing does exists
-    unless( exists($heap->{things}->{$key}) ) {
-            $kernel->post( $service, 'FAULT', $response, HTTP_BAD_REQUEST, 'EXISTS', "Thing $key does not exist." );
+    # Check if referenced author does exists
+    unless( exists($heap->{authors}->{$key}) ) {
+            $kernel->post( $service, 'FAULT', $response, HTTP_BAD_REQUEST, 'EXISTS', "Author $key does not exist." );
             return;
     }
 
 	# Delete it	
-	delete $heap->{things}->{$key};
+	delete $heap->{authors}->{$key};
 
 	# Done
-	$kernel->post( $service, 'DONE', $response, "Done", "Successfully removed ththingg." );
+	$kernel->post( $service, 'DONE', $response, "Done", "Successfully removed the author." );
 	return;
 
 }
